// src/index.ts
import { mkdir, readFile, writeFile } from "fs/promises";
import { resolve, dirname } from "path";

// src/core.ts
import { base64 } from "rfc4648";

// src/crypto.ts
async function loadCrypto() {
  if (typeof window !== "undefined" && window.crypto || globalThis && globalThis.crypto) {
    const crypto2 = window.crypto || globalThis.crypto;
    return new Promise((resolve2) => resolve2(crypto2));
  } else {
    const nodeCrypto = await import("crypto");
    return nodeCrypto.webcrypto;
  }
}
var crypto = await loadCrypto();
var crypto_default = crypto;

// src/decrypt-template.html
var decrypt_template_default = `<!DOCTYPE html>\r
<html>\r
<head>\r
    <meta charset="utf-8">\r
    <meta name="viewport" content="width=device-width, initial-scale=1">\r
    <meta name="robots" content="noindex, nofollow">\r
    <title>Protected Page</title>\r
    <script type="module">var e,t;var n={chars:"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",bits:6},a=function(e,t){return function(e,t,n){var a;if(void 0===n&&(n={}),!t.codes){t.codes={};for(var r=0;r<t.chars.length;++r)t.codes[t.chars[r]]=r}if(!n.loose&&e.length*t.bits&7)throw new SyntaxError("Invalid padding");for(var i=e.length;"="===e[i-1];)if(--i,!(n.loose||(e.length-i)*t.bits&7))throw new SyntaxError("Invalid padding");for(var o=new(null!=(a=n.out)?a:Uint8Array)(i*t.bits/8|0),s=0,c=0,d=0,l=0;l<i;++l){var u=t.codes[e[l]];if(void 0===u)throw new SyntaxError("Invalid character "+e[l]);c=c<<t.bits|u,(s+=t.bits)>=8&&(s-=8,o[d++]=255&c>>s)}if(s>=t.bits||255&c<<8-s)throw new SyntaxError("Unexpected end of data");return o}(e,n,t)};const r=document.querySelector.bind(document),[i,o,s,c,d]=["input","header","#msg","form","#load"].map(r);let l,u,w;document.addEventListener("DOMContentLoaded",(async()=>{const e=r("pre").innerText;if(!e)return i.disabled=!0,void v("No encrypted payload.");const t=a(e);l=t.slice(0,32),u=t.slice(32,48),w=t.slice(48),location.hash&&(i.value=location.hash.slice(1),history.replaceState(null,"",".")),sessionStorage.k||i.value?await p():(f(d),y(c),o.classList.replace("hidden","flex"),i.focus())}));const h=(null==(e=window.crypto)?void 0:e.subtle)||(null==(t=window.crypto)?void 0:t.webkitSubtle);function y(e){e.classList.remove("hidden")}function f(e){e.classList.add("hidden")}function v(e){s.innerText=e,o.classList.add("text-red-600")}async function p(){d.lastElementChild.innerText="Decrypting...",f(o),f(c),y(d),await async function(e){return new Promise((t=>setTimeout(t,e)))}(60);try{const e=await async function({salt:e,iv:t,ciphertext:n},a){const r=new TextDecoder,i=sessionStorage.k?await async function(e){return h.importKey("jwk",e,"AES-GCM",!0,["decrypt"])}(JSON.parse(sessionStorage.k)):await async function(e,t){const n=new TextEncoder,a=await h.importKey("raw",n.encode(t),"PBKDF2",!1,["deriveKey"]);return await h.deriveKey({name:"PBKDF2",salt:e,iterations:2e6,hash:"SHA-256"},a,{name:"AES-GCM",length:256},!0,["decrypt"])}(e,a),o=new Uint8Array(await h.decrypt({name:"AES-GCM",iv:t},i,n));if(!o)throw"Malformed data";return sessionStorage.k=JSON.stringify(await h.exportKey("jwk",i)),r.decode(o)}({salt:l,iv:u,ciphertext:w},i.value);document.write(e),document.close()}catch(e){f(d),y(c),o.classList.replace("hidden","flex"),sessionStorage.k?sessionStorage.removeItem("k"):v("Wrong password."),i.value="",i.focus()}}h||(v("Please use a modern browser."),i.disabled=!0),c.addEventListener("submit",(async e=>{e.preventDefault(),await p()}));<\/script>\r
    <style>/*! tailwindcss v2.2.7 | MIT License | https://tailwindcss.com *//*! modern-normalize v1.1.0 | MIT License | https://github.com/sindresorhus/modern-normalize */*,::after,::before{box-sizing:border-box}html{-moz-tab-size:4;-o-tab-size:4;tab-size:4}html{line-height:1.15;-webkit-text-size-adjust:100%}body{margin:0}body{font-family:system-ui,-apple-system,'Segoe UI',Roboto,Helvetica,Arial,sans-serif,'Apple Color Emoji','Segoe UI Emoji'}hr{height:0;color:inherit}abbr[title]{-webkit-text-decoration:underline dotted;text-decoration:underline dotted}b,strong{font-weight:bolder}code,kbd,pre,samp{font-family:ui-monospace,SFMono-Regular,Consolas,'Liberation Mono',Menlo,monospace;font-size:1em}small{font-size:80%}sub,sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}sub{bottom:-.25em}sup{top:-.5em}table{text-indent:0;border-color:inherit}button,input,optgroup,select,textarea{font-family:inherit;font-size:100%;line-height:1.15;margin:0}button,select{text-transform:none}[type=button],[type=reset],[type=submit],button{-webkit-appearance:button}::-moz-focus-inner{border-style:none;padding:0}:-moz-focusring{outline:1px dotted ButtonText}:-moz-ui-invalid{box-shadow:none}legend{padding:0}progress{vertical-align:baseline}::-webkit-inner-spin-button,::-webkit-outer-spin-button{height:auto}[type=search]{-webkit-appearance:textfield;outline-offset:-2px}::-webkit-search-decoration{-webkit-appearance:none}::-webkit-file-upload-button{-webkit-appearance:button;font:inherit}summary{display:list-item}blockquote,dd,dl,figure,h1,h2,h3,h4,h5,h6,hr,p,pre{margin:0}button{background-color:transparent;background-image:none}fieldset{margin:0;padding:0}ol,ul{list-style:none;margin:0;padding:0}html{font-family:ui-sans-serif,system-ui,-apple-system,BlinkMacSystemFont,"Segoe UI",Roboto,"Helvetica Neue",Arial,"Noto Sans",sans-serif,"Apple Color Emoji","Segoe UI Emoji","Segoe UI Symbol","Noto Color Emoji";line-height:1.5}body{font-family:inherit;line-height:inherit}*,::after,::before{box-sizing:border-box;border-width:0;border-style:solid;border-color:currentColor}hr{border-top-width:1px}img{border-style:solid}textarea{resize:vertical}input::-moz-placeholder,textarea::-moz-placeholder{opacity:1;color:#9ca3af}input:-ms-input-placeholder,textarea:-ms-input-placeholder{opacity:1;color:#9ca3af}input::placeholder,textarea::placeholder{opacity:1;color:#9ca3af}[role=button],button{cursor:pointer}table{border-collapse:collapse}h1,h2,h3,h4,h5,h6{font-size:inherit;font-weight:inherit}a{color:inherit;text-decoration:inherit}button,input,optgroup,select,textarea{padding:0;line-height:inherit;color:inherit}code,kbd,pre,samp{font-family:ui-monospace,SFMono-Regular,Menlo,Monaco,Consolas,"Liberation Mono","Courier New",monospace}audio,canvas,embed,iframe,img,object,svg,video{display:block;vertical-align:middle}img,video{max-width:100%;height:auto}[hidden]{display:none}*,::after,::before{--tw-border-opacity:1;border-color:rgba(229,231,235,var(--tw-border-opacity));--tw-ring-offset-shadow:0 0 #0000;--tw-ring-shadow:0 0 #0000;--tw-shadow:0 0 #0000}.fixed{position:fixed}.bottom-0{bottom:0}.right-0{right:0}.mx-auto{margin-left:auto;margin-right:auto}.mr-4{margin-right:1rem}.mb-4{margin-bottom:1rem}.flex{display:flex}.hidden{display:none}.h-screen{height:100vh}.h-28{height:7rem}.h-full{height:100%}.h-6{height:1.5rem}.w-full{width:100%}.w-6{width:1.5rem}.max-w-sm{max-width:24rem}.items-start{align-items:flex-start}.items-center{align-items:center}.justify-center{justify-content:center}.gap-2{gap:.5rem}.rounded-sm{border-radius:.125rem}.border{border-width:1px}.bg-yellow-200{--tw-bg-opacity:1;background-color:rgba(253,230,138,var(--tw-bg-opacity))}.bg-gray-50{--tw-bg-opacity:1;background-color:rgba(249,250,251,var(--tw-bg-opacity))}.bg-gray-200{--tw-bg-opacity:1;background-color:rgba(229,231,235,var(--tw-bg-opacity))}.p-4{padding:1rem}.py-2{padding-top:.5rem;padding-bottom:.5rem}.px-4{padding-left:1rem;padding-right:1rem}.pt-40{padding-top:10rem}.text-sm{font-size:.875rem;line-height:1.25rem}.font-light{font-weight:300}.font-extralight{font-weight:200}.font-semibold{font-weight:600}.tracking-wide{letter-spacing:.025em}.tracking-wider{letter-spacing:.05em}.text-black{--tw-text-opacity:1;color:rgba(0,0,0,var(--tw-text-opacity))}.text-opacity-100{--tw-text-opacity:1}.placeholder-black::-moz-placeholder{--tw-placeholder-opacity:1;color:rgba(0,0,0,var(--tw-placeholder-opacity))}.placeholder-black:-ms-input-placeholder{--tw-placeholder-opacity:1;color:rgba(0,0,0,var(--tw-placeholder-opacity))}.placeholder-black::placeholder{--tw-placeholder-opacity:1;color:rgba(0,0,0,var(--tw-placeholder-opacity))}.shadow-2xl{--tw-shadow:0 25px 50px -12px rgba(0, 0, 0, 0.25);box-shadow:var(--tw-ring-offset-shadow,0 0 #0000),var(--tw-ring-shadow,0 0 #0000),var(--tw-shadow)}.loading{pointer-events:none;width:2em;height:2em;border:.15em solid transparent;border-color:#e2e8f0;border-top-color:#2563eb;border-radius:50%;-webkit-animation:spin .5s linear infinite;animation:spin .5s linear infinite}@-webkit-keyframes spin{100%{transform:rotate(360deg)}}@keyframes spin{100%{transform:rotate(360deg)}}.focus\\:border-black:focus{--tw-border-opacity:1;border-color:rgba(0,0,0,var(--tw-border-opacity))}.focus\\:outline-none:focus{outline:2px solid transparent;outline-offset:2px}</style>\r
  </head>\r
<body>\r
    <main class="bg-yellow-200 w-full h-screen items-start tracking-wide p-4 pt-40 font-light">\r
        <div class="max-w-sm w-full bg-gray-50 p-4 rounded-sm shadow-2xl mx-auto h-28">\r
            <div id="load" class="flex items-center justify-center h-full">\r
                <p class="loading w-6 h-6 mr-4"></p><p>Loading...</p>\r
            </div>\r
            <header class="hidden gap-2 mb-4 items-center">\r
                <svg id="locked" class="w-6 h-6" fill="currentColor" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg">\r
                    <path fill-rule="evenodd" d="M5 9V7a5 5 0 0110 0v2a2 2 0 012 2v5a2 2 0 01-2 2H5a2 2 0 01-2-2v-5a2 2 0 012-2zm8-2v2H7V7a3 3 0 016 0z" clip-rule="evenodd"></path>\r
                </svg>\r
                <p id="msg" class="text-sm">This page is password protected.</p>\r
            </header>\r
            <form class="hidden">\r
                <input type="password" id="pwd" name="pwd" aria-label="Password" autofocus placeholder="Password" class="font-extralight bg-gray-200 flex w-full py-2 px-4 tracking-wider rounded-sm focus:outline-none text-black placeholder-black text-opacity-100 focus:border-black border" />\r
            </form>\r
        </div>\r
        <a href="https://github.com/Greenheart/pagecrypt" class="fixed bottom-0 right-0 p-4">Created with <span class="font-semibold">PageCrypt</span></a>\r
    </main>\r
    <!--ENCRYPTED PAYLOAD-->\r
</body>\r
</html>`;

// src/core.ts
async function getEncryptedPayload(content, password) {
  const encoder = new TextEncoder();
  const salt = crypto_default.getRandomValues(new Uint8Array(32));
  const baseKey = await crypto_default.subtle.importKey("raw", encoder.encode(password), "PBKDF2", false, ["deriveKey"]);
  const key = await crypto_default.subtle.deriveKey({ name: "PBKDF2", salt, iterations: 2e6, hash: "SHA-256" }, baseKey, { name: "AES-GCM", length: 256 }, false, ["encrypt"]);
  const iv = crypto_default.getRandomValues(new Uint8Array(16));
  const ciphertext = new Uint8Array(await crypto_default.subtle.encrypt({ name: "AES-GCM", iv }, key, encoder.encode(content)));
  const totalLength = salt.length + iv.length + ciphertext.length;
  const mergedData = new Uint8Array(totalLength);
  mergedData.set(salt);
  mergedData.set(iv, salt.length);
  mergedData.set(ciphertext, salt.length + iv.length);
  return base64.stringify(mergedData);
}
async function encryptHTML(inputHTML, password) {
  return decrypt_template_default.replace(/<!--ENCRYPTED PAYLOAD-->/, `<pre class="hidden">${await getEncryptedPayload(inputHTML, password)}</pre>`);
}
function generatePassword(length = 80, characters = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz") {
  return Array.from({ length }, (_) => getRandomCharacter(characters)).join("");
}
function getRandomCharacter(characters) {
  let randomNumber;
  do {
    randomNumber = crypto_default.getRandomValues(new Uint8Array(1))[0];
  } while (randomNumber >= 256 - 256 % characters.length);
  return characters[randomNumber % characters.length];
}

// src/index.ts
async function encryptFile(inputFile, password) {
  let content;
  try {
    content = await readFile(resolve(process.cwd(), inputFile), {
      encoding: "utf-8"
    });
  } catch (e) {
    console.error("\u274C Error reading file: ", e);
    process.exit(1);
  }
  return await encryptHTML(content, password);
}
async function saveFile(outputFile, content) {
  await mkdir(dirname(outputFile), { recursive: true });
  return writeFile(resolve(process.cwd(), outputFile), content, {
    encoding: "utf8"
  });
}
async function encrypt(inputFile, outputFile, password) {
  const encrypted = await encryptFile(inputFile, password);
  return await saveFile(outputFile, encrypted);
}
export {
  encrypt,
  encryptHTML,
  generatePassword
};
